import matplotlib.pyplot as plt
import os
import numpy as np
import os

dir="1000mrnas"


def find_nth(string, substring, n):
   if (n == 1):
       return string.find(substring)
   else:
       return string.find(substring, find_nth(string, substring, n - 1) + 1)
#plot current-density relation (non normalized)
file1="alpha_rho_J"
data=[]
geneIndex=[]
directory = os.fsencode(dir+'/'+file1)
for file in os.listdir(directory):
    filename = os.fsdecode(file)
    p1=filename.index("_")
    p2=find_nth(filename,"_",2)
    index=int(filename[p1+1:p2])
    if index not in geneIndex:
        geneIndex.append(index)
for i in range(max(geneIndex)+1):
    data.append([])
for file in os.listdir(directory):
    filename = os.fsdecode(file)
    b=filename.index("beta")
    beta=float(filename[b+5:b+10])

    with open(dir+'/'+file1+"/"+filename) as my_file:
        line = my_file.readlines()[0]
        ar=line.split("\t")
        l=[float(ar[0]),beta,float(ar[1]),float(ar[2]),float(ar[3]),int(ar[4])] #alpha, beta, rho, j,meankRate,+++numberOfproteins
    p1=filename.index("_")
    p2=find_nth(filename,"_",2)
    index=int(filename[p1+1:p2])
    data[index].append(l)

#data is a matrix of dimension M(number of mrnas) x [alph,beta] (number of alpha beta value pairs) x 5
# containing in each row [alpha,beta,menaRho,meanElongationRate]
#data[0] has the data for first mrna
#data[0][0] corresponds to the first alpha,beta value pair for the first mrna
#data[0][0][3], gives back the mean rho value.


x_values = np.linspace(0, 1, 100) #to plot the mena_field results of 'TASEP
y_values = []
for x in x_values:
    y_values.append(x*(1-x))

#prepare data to plot a histogram of the number of produced proteins per mrna, the same plot should contain the mrna sizes.
numberOfproteins=[]
for d in data:
    #print(len(d))
    numberOfproteins.append(d[2][5])
percentageofprotein=[]
for x in numberOfproteins:
    percentageofprotein.append((x*100)/sum(numberOfproteins))
#max_rows=50
sizes=np.loadtxt("10mrnas/C_sizes_mRNAs.dat",max_rows=len(data))
fig, ax1 = plt.subplots()
ax1.set_xlabel('mRNA',fontsize=16,color="blue")
ax1.set_ylabel('% of produced proteins',fontsize=12,color="blue")
ax1.set_ylim(0,20)
ax1.tick_params(axis='y', labelcolor="blue")
ax2 = ax1.twinx()
ax2.set_ylabel("mRNA size in codons",fontsize=12,color="red")
ax2.tick_params(axis='y', labelcolor="red")
line2, = ax2.plot( range(len(data)), sizes,marker=".",label="size of mRNA in codons",linestyle ="dotted",color="red",markersize=5)
line1, = ax1.plot( range(len(data)), percentageofprotein,color="blue",label="Protein production in %",linestyle="-",marker="*")
plt.legend(handles=[line1,line2],fontsize=14)
plt.savefig(dir+"/res.jpg",dpi=300)
plt.clf()



for i in range(len(data)): #for each mrna
    #plot particle density as heatmaps
    array1=np.array(data[i])
    sc=plt.scatter(array1[:,0], array1[:,1], c=array1[:,2]*10,cmap="RdYlGn")
    #plt.show()
    plt.colorbar(sc)
    plt.xlabel(r'$\alpha$')
    plt.ylabel(r'$\beta$')
    plt.title("Particle Density on gene "+str(i))
    plt.savefig(dir+'/'+"mean_density_gene"+str(i)+".jpg",dpi=300)
    #plt.savefig(dir+'/'+"mean_density_gene"+str(i)+".eps",dpi=300)
    plt.clf()

    #normalize alpha and beta by mean-k and plot
    sc=plt.scatter(array1[:,0]/array1[:,4], array1[:,1]/array1[:,4], c=array1[:,2]*10,cmap="RdYlGn")
    plt.colorbar(sc)
    plt.xlabel(r'$\alpha/meanElongation$')
    plt.ylabel(r'$\beta/meanElongation$')
    plt.title("Particle Density on gene "+str(i))
    plt.savefig(dir+'/'+"norm.mean_density_gene"+str(i)+".jpg",dpi=300)
    #plt.savefig(dir+'/'+"norm.mean_density_gene"+str(i)+".eps",dpi=300)
    plt.clf()

    #### plot j(rho)
    plt.xlabel(r'$\rho$')
    plt.ylabel('J')
    plt.xlim(-0.05,1.05)
    plt.plot(x_values, y_values, color="yellow",label=r'$f(x) = x(1-x)$')
    plt.plot(array1[:,2]*10,array1[:,3],color="black",label="J(rho)",linestyle="None",marker=".")
    plt.legend()
    plt.savefig(dir+'/'+"J(rho)_gene"+str(i)+".jpg",dpi=300)
    #plt.savefig(dir+'/'+"J(rho)_gene"+str(i)+".eps",dpi=300)
    plt.clf()

    ### plot J/meanElongation(rho)
    plt.xlabel(r'$\rho$')
    plt.ylabel('J/meanElongation')
    plt.xlim(-0.05,1.05)
    plt.plot(x_values, y_values, color="yellow",label=r'$f(x) = x(1-x)$')
    plt.plot(array1[:,2]*10,array1[:,3]/array1[:,4],color="blue",label="J/meanK(rho)",linestyle="None",marker=".")
    plt.legend()
    plt.savefig(dir+'/'+"norm.J(rho)_gene"+str(i)+".jpg",dpi=300)
    #plt.savefig(dir+'/'+"norm.J(rho)_gene"+str(i)+".eps",dpi=300)
    plt.clf()

    ###Plot number of proteins


#mean charging levels and number of charged tRNAS
file1="mean_charging"
if not os.path.exists(dir+"/"+file1+"/fig"):
    os.makedirs(dir+"/"+file1+"/fig")
directory = os.fsencode(dir+'/'+file1)
print("plot charging dynamics ...")
for file in os.listdir(directory):
    charg=[]
    filename = os.fsdecode(file)
    if not os.path.isdir(dir+'/'+file1+"/"+filename):
        with open(dir+'/'+file1+"/"+filename) as my_file:
            for line in my_file.readlines():
                ar=line.split("\t")
                charg.append([float(ar[0]),float(ar[1]),float(ar[2])])
        charg=np.array(charg)
        #charg has the contaent of one file as an array
        plt.xlabel("tRNA index")
        plt.ylabel('Charging level')
        plt.ylim(-0.05,1.05)
        #plt.plot(x_values, y_values, color="yellow",label=r'$f(x) = x(1-x)$')
        plt.plot(charg[:,0],charg[:,1],color="black",label="charging level",linestyle="-",marker=".")
        #plt.plot(charg[:,0],charg[:,2],color="red",label="mean number of charged tRNAs",linestyle="None",marker=".")
        plt.legend()
        plt.savefig(dir+"/"+file1+"/fig/level_"+filename+".jpg",dpi=300)
        #plt.savefig(dir+"/"+file1+"/fig/"+filename+".eps",dpi=300)
        plt.clf()

        plt.xlabel("tRNA index")
        plt.ylabel('Charging level')
        #plt.xlim(-0.05,1.05)
        #plt.plot(x_values, y_values, color="yellow",label=r'$f(x) = x(1-x)$')
        #plt.plot(charg[:,0],charg[:,1],color="black",label="charging level",linestyle="-",marker=".")
        plt.plot(charg[:,0],charg[:,2],color="red",label="mean number of charged tRNAs",linestyle="-",marker=".")
        plt.legend()
        plt.savefig(dir+"/"+file1+"/fig/number_"+filename+".jpg",dpi=300)
        #plt.savefig(dir+"/"+file1+"/fig/"+filename+".eps",dpi=300)
        plt.clf()

#Density profiles to have a percentage of the occupation of the sites along the mrna
print("plot density profiles...")
file1="density_profile"
if not os.path.exists(dir+"/"+file1+"/fig"):
    os.makedirs(dir+"/"+file1+"/fig")
directory = os.fsencode(dir+'/'+file1)

for file in os.listdir(directory):
    filename = os.fsdecode(file)
    if not os.path.isdir(dir+'/'+file1+"/"+filename):
        data=np.loadtxt(dir+'/'+file1+"/"+filename)
        data=np.array(data)
        plt.xlabel("mRNA codons")
        plt.ylabel("mean Density")
        plt.ylim(-0.01,1.0)
        plt.plot(data[:,0],data[:,1], color="black", marker=".")
        plt.savefig(dir+"/"+file1+"/fig/"+filename+".jpg",dpi=300)
        plt.clf()

#TODO : take just the alpha beta =0.8 both, and plot number of ribosomes on the mrna and %of occupied codons compared to the size
file1="RPF"
if not os.path.exists(dir+"/"+file1+"/fig"):
    os.makedirs(dir+"/"+file1+"/fig")
directory = os.fsencode(dir+'/'+file1)

numberOfribs=[]
with open (dir+"/RPF/alpha_109515838.037728_beta_0.200000.dat","r")as f:
    lines=f.readlines()
    for line in lines:
        numberOfribs.append(len(line.split("\t"))-1)
print(numberOfribs)
#plot number of ribosomes per mrna with the % of covered codons.
fig, ax1 = plt.subplots()
ax1.set_xlabel('mRNA',fontsize=16,color="orange")
ax1.set_ylabel('number of ribosomes',fontsize=12,color="orange")
ax1.set_ylim(0,100)
ax1.tick_params(axis='y', labelcolor="orange")
ax2 = ax1.twinx()
ax2.set_ylabel("% of covered codons",fontsize=12,color="green")
ax2.set_ylim(0,100)
ax2.tick_params(axis='y', labelcolor="green")
line2, = ax2.plot( range(len(numberOfribs)), (numberOfribs/sizes)*1000,marker=".",label="covered codons in %",linestyle ="dotted",color="green",markersize=5)
line1, = ax1.plot( range(len(numberOfribs)), numberOfribs,color="orange",label="number of ribosomes per mRNA",linestyle="-",marker="*")
plt.legend(handles=[line1,line2],fontsize=14)
plt.savefig(dir+"/res1.jpg",dpi=300)
plt.clf()



'''
for file in os.listdir(directory):
    filename = os.fsdecode(file)
    if not os.path.isdir(dir+'/'+file1+"/"+filename):
        with open(dir+'/'+file1+"/"+filename) as f:
            lines=f.readlines()
            index=-1
            s=""
            for line in lines:
                index+=1  #index of the mrna
                #take into account that ribosomes occupy 4 sites before particle and 5 afterwards
                array=np.array(line.split("\t"))
                array[array == '\n'] = 0
                array=array.astype(int)
                for j in array:
                    if j !=0:
                        for jj in range(5):
                            array=np.append(array,j-jj)
                            array=np.append(array,j+jj)
                        array=np.append(array,j+5)
                print(array)
                for i in range(int(sizes[index])):
                    if i in array:
                        s=s+"*"
                    else:
                        s=s+"-"
                print(s)

'''
#mean charging levels and number of charged tRNAS
file1="mean_charging"
if not os.path.exists(dir+"/"+file1+"/fig"):
    os.makedirs(dir+"/"+file1+"/fig")
directory = os.fsencode(dir+'/'+file1)
print("plot charging dynamics ...")
for file in os.listdir(directory):
    charg=[]
    filename = os.fsdecode(file)
    if not os.path.isdir(dir+'/'+file1+"/"+filename):
        with open(dir+'/'+file1+"/"+filename) as my_file:
            for line in my_file.readlines():
                ar=line.split("\t")
                charg.append([float(ar[0]),float(ar[1]),float(ar[2])])
        charg=np.array(charg)
        #charg has the contaent of one file as an array
        plt.xlabel("tRNA index")
        plt.ylabel('Charging level')
        plt.ylim(-0.05,1.05)
        #plt.plot(x_values, y_values, color="yellow",label=r'$f(x) = x(1-x)$')
        plt.plot(charg[:,0],charg[:,1],color="black",label="charging level",linestyle="-",marker=".")
        #plt.plot(charg[:,0],charg[:,2],color="red",label="mean number of charged tRNAs",linestyle="None",marker=".")
        plt.legend()
        plt.savefig(dir+"/"+file1+"/fig/level_"+filename+".jpg",dpi=300)
        #plt.savefig(dir+"/"+file1+"/fig/"+filename+".eps",dpi=300)
        plt.clf()

        plt.xlabel("tRNA index")
        plt.ylabel('Charging level')
        #plt.xlim(-0.05,1.05)
        #plt.plot(x_values, y_values, color="yellow",label=r'$f(x) = x(1-x)$')
        #plt.plot(charg[:,0],charg[:,1],color="black",label="charging level",linestyle="-",marker=".")
        plt.plot(charg[:,0],charg[:,2],color="red",label="mean number of charged tRNAs",linestyle="-",marker=".")
        plt.legend()
        plt.savefig(dir+"/"+file1+"/fig/number_"+filename+".jpg",dpi=300)
        #plt.savefig(dir+"/"+file1+"/fig/"+filename+".eps",dpi=300)
        plt.clf()


